@@ -4,6 +4,7 @@ import configparser
 from dateutil.parser import parse
 from datetime import datetime, timedelta
 import requests
+from requests.structures import CaseInsensitiveDict
 from gql import gql, Client
 from gql.transport.requests import RequestsHTTPTransport
 import io
@@ -17,6 +18,7 @@ import pkg_resources
 import urllib3
 import backoff
 import logging
+from notebook.notebookapp import list_running_servers
 
 urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
 
@@ -25,9 +27,7 @@ from ocs_sample_library_preview import OCSClient, DataView, SdsError
 
 UXIE_CONSTANT = 100 * 1000
 HUB_BLOB = "https://academichub.blob.core.windows.net/hub/"
-BINDER_INI = HUB_BLOB + "config-binder.ini"
 COLLAB_INI = HUB_BLOB + "config-collab.ini"
-GITHUB_HUB_DATASETS = "https://github.com/academic-hub/datasets"
 
 hub_db_namespaces = {}
 
@@ -116,18 +116,12 @@ class HubClient(OCSClient):
         config_filename = os.environ.get("OCS_HUB_CONFIG", None)
         config_file = None
         if config_filename is None:
-            binder_repo = os.environ.get("BINDER_REPO_URL", None)
-            if binder_repo is not None:
-                print("@ --- OSIsoft authorization required to run on Binder ---")
-                if binder_repo == GITHUB_HUB_DATASETS:
-                    config_file = get_config(BINDER_INI)
-                else:
-                    print("@ ### authorization denied ###")
-                    return
             if len(collab_key) > 0:
                 print("@ --- OSIsoft authorization required to run on Collab ---")
                 config_file = get_config(COLLAB_INI, client_secret=collab_key)
-
+            elif next(list_running_servers())["hostname"] != "localhost":
+                if os.path.exists("config.txt"):
+                    config_filename = "config.txt"
         if config_filename is None and config_file is None:
             super().__init__(
                 "v1",
@@ -292,7 +286,11 @@ class HubClient(OCSClient):
     )
     @typechecked
     def dataview_definition(
-        self, namespace_id: str, dataview_id: str, stream_id=False, version: str = ""
+        self,
+        namespace_id: str,
+        dataview_id: str,
+        stream_id: bool = False,
+        version: str = "",
     ):
         columns = [
             "Asset_Id",
@@ -304,6 +302,7 @@ class HubClient(OCSClient):
         if stream_id:
             columns += ["OCS_Stream_Id"]
         df = pd.DataFrame(columns=columns)
+
         data_items = super().DataViews.getResolvedDataItems(
             namespace_id, dataview_id, "Asset_value?count=1000&cache=refresh"
         )
@@ -312,7 +311,7 @@ class HubClient(OCSClient):
             "column_name" if v2_column_key is None else f"{v2_column_key}|column"
         )
         for i, item in enumerate(data_items.Items):
-            item_meta = self.__asdict(item.Metadata)
+            item_meta = CaseInsensitiveDict(self.__asdict(item.Metadata))
             column_values = [
                 item_meta["asset_id"],
                 item_meta[column_key],
@@ -372,6 +371,27 @@ class HubClient(OCSClient):
             **count_arg,
         )
 
+    def __get_data_stored(
+        self,
+        namespace_id,
+        dataview_id,
+        form,  # not used
+        start_index,
+        end_index,
+        interval,  # not used
+        count,
+        next_page,
+    ):
+        count_arg = {} if count is None else {"count": count}
+        return super().DataViews.getDataStored(
+            namespace_id,
+            dataview_id,
+            startIndex=start_index,
+            endIndex=end_index,
+            url=next_page,
+            **count_arg,
+        )
+
     @backoff.on_exception(
         backoff.expo, SdsError50x, max_tries=6, jitter=backoff.full_jitter
     )
@@ -387,9 +407,34 @@ class HubClient(OCSClient):
         count: int = None,
         sub_second_interval: bool = False,
         verbose: bool = False,
+        stored: bool = False,
+    ):
+        return self.dataview_get_data_pd(
+            namespace_id,
+            dataview_id,
+            start_index,
+            end_index,
+            interval,
+            count,
+            sub_second_interval,
+            verbose,
+            stored,
+        )
+
+    def dataview_get_data_pd(
+        self,
+        namespace_id: str,
+        dataview_id: str,
+        start_index: str,
+        end_index: str,
+        interval: str,
+        count: int = None,
+        sub_second_interval: bool = False,
+        verbose: bool = False,
+        stored: bool = False,
     ):
         df = pd.DataFrame()
-        if not sub_second_interval:
+        if not sub_second_interval and not stored:
             try:
                 datetime.strptime(interval, "%H:%M:%S")
             except ValueError as e:
@@ -405,16 +450,16 @@ class HubClient(OCSClient):
             summary = f"<@dataview_interpolated_pd/{dataview_id}/{start_index}/{end_index}/{interval}  t={datetime.now().isoformat()}"
             print(summary)
 
-        interpolated_f = (
-            self.__get_data_interpolated_gql
-            if "cache" in self.__options
-            else self.__get_data_interpolated
-        )
+        dataview_f = self.__get_data_stored if stored else self.__get_data_interpolated
+        # when GraphQL interface is in place
+        # self.__get_data_interpolated_gql
+        # if "cache" in self.__options
+
         dataview_id = self.remap_campus_dataview_id(dataview_id)
         next_page = None
         while True:
             try:
-                csv, next_page, _ = interpolated_f(
+                csv_or_json, next_page, _ = dataview_f(
                     namespace_id=namespace_id,
                     dataview_id=dataview_id,
                     count=count,
@@ -424,11 +469,17 @@ class HubClient(OCSClient):
                     interval=interval,
                     next_page=next_page,
                 )
-                # print(f"[{len(csv)}]", end="")
-                df = df.append(
-                    pd.read_csv(io.StringIO(csv), parse_dates=["Timestamp"]),
-                    ignore_index=True,
-                )
+                # print(f"[{len(csv)}, {type(csv)}, {str(csv)}]", end="")
+                if not stored:
+                    df = df.append(
+                        pd.read_csv(
+                            io.StringIO(csv_or_json), parse_dates=["Timestamp"]
+                        ),
+                        ignore_index=True,
+                    )
+                else:
+                    df = df.append(pd.read_json(json.dumps(csv_or_json)))
+
                 if next_page is None:
                     print()
                     break
@@ -457,6 +508,40 @@ class HubClient(OCSClient):
 
         return self.__process_digital_states(df)
 
+    @backoff.on_exception(
+        backoff.expo, SdsError50x, max_tries=6, jitter=backoff.full_jitter
+    )
+    @timer
+    @typechecked
+    def dataview_stored_pd(
+        self,
+        namespace_id: str,
+        dataview_id: str,
+        start_index: str,
+        end_index: str,
+        count: int = None,
+    ):
+        try:
+            result = self.dataview_get_data_pd(
+                namespace_id,
+                dataview_id + "_narrow",
+                start_index,
+                end_index,
+                "",
+                count=count,
+                stored=True,
+            )
+        except SdsError as e:
+            if "404" in str(e):
+                print(
+                    f"### Error: data view with Id {dataview_id} has no version for stored data.\n"
+                    "###  If data view id is correct, contact Hub support if stored data is required instead of interpolated."
+                )
+                return
+            else:
+                raise e
+        return result
+
     def request(self, method, url, params=None, data=None, headers=None, **kwargs):
         print(dir(self))
         return self._OCSClient__baseClient.request(
